/* Abstract Syntax Notation One (ASN.1) Distinguished Encoding Rules (DER)
   encoding for RSA/MD5, used in PKCS-format signatures. */
static byte asn_array[] = { /* PKCS 01 block type 01 data */
    0x30,0x20,0x30,0x0c,0x06,0x08,0x2a,0x86,0x48,0x86,0xf7,0x0d,
    0x02,0x05,0x05,0x00,0x04,0x10 };
/*
   md5 OBJECT IDENTIFIER ::=
           {iso(1) member-body(2) US(840) rsadsi(113549)
            digestAlgorithm(2) 5}
*/

static byte ripemd160_asn_array[] = {
    0x30,0x21,0x30,0x09,0x06,0x05,0x2B,0x24,0x03,0x02,0x01,
    0x05,0x00,0x04,0x14 };
/*
         RIPEMD-160 OBJECT IDENTIFIER ::= {
            iso(1) identified-organization(3) TeleTrusT(36) algorithm(3) 
            hashAlgorithm(2) 26 
         }
*/

static byte sha1_asn_array[] = {    
    0x30,0x21,0x30,0x09,0x06,0x05,0x2b,0x0e,0x03,0x02,0x1a,
    0x05,0x00,0x04,0x14 };
/*
Taken from Internet Draft draft-ietf-cat-spkmgss-06,
"The Simple Public-Key GSS-API Mechanism (SPKM)", by
C. Adams, Bell-Northern Research, Jan. 19, 1996.  See
also "Working Implementation Agreements for Open Systems
Interconnection Protocols: Part 12 - OS Security, Output
from  the   December  1994   Open  Systems   Environment
Implementors' Workshop (OIW)" 

         SHA1 OBJECT IDENTIFIER ::= {
            iso(1) identified-organization(3) oiw(14) secsig(3) 
            algorithm(2) 26 
         }

ASN.1 encoding:
  0x30, / * Universal, Constructed, Sequence * /
        0x21, / * Length 33 (bytes following) * /
                0x30, / * Universal, Constructed, Sequence * /
                0x09, / * Length 9 * /
                        0x06, / * Universal, Primitive, object-identifier * /
                        0x05, / * Length 5 * /
                                43, / * 43 = ISO(1)*40 + 3 * /
                                14,
                                3,
                                2,
                                26,
                        0x05, / * Universal, Primitive, NULL * /
                        0x00, / * Length 0 * /
                0x04, / * Universal, Primitive, Octet string * /
                0x14 / * Length 20 * /
                        / * 20 SHA.1 digest bytes go here * /

Cf. "A Layman's Guide to a Subset of ASN.1, BER, and DER --
An RSA Laboratories Technical Note" by Burton S. Kaliski Jr.
Revised November 1, 1993
*/

#define sha1x_asn_array sha1_asn_array /* this is wrong... but compatible with CKT */

static byte sha256_asn_array[] = {
    0x30,0x31,0x30,0x0d,0x06,0x09,0x60,0x86,0x48,
    0x01,0x65,0x03,0x04,0x02,0x01,0x05,0x00,0x04,0x20};
static byte sha384_asn_array[] = {
    0x30,0x41,0x30,0x0d,0x06,0x09,0x60,0x86,0x48,
    0x01,0x65,0x03,0x04,0x02,0x02,0x05,0x00,0x04,0x30};
static byte sha512_asn_array[] = {
    0x30,0x51,0x30,0x0d,0x06,0x09,0x60,0x86,0x48,
    0x01,0x65,0x03,0x04,0x02,0x03,0x05,0x00,0x04,0x40};
/*
         2.16.840.1.101.3.4.2.1
         SHA256 OBJECT IDENTIFIER ::= {
            joint-iso-ccitt(2) country(16) us(840) organization(1)
            us-government(101) csor(3)
            nistAlgorithm(4) (2) 1
         }
*/

/* 40: SEQUENCE {
     * 12:   SEQUENCE {
     *	8:     OCTET STRING   :54 49 47 45 52 31 39 32
     *	0:     NULL
     *	 :     }
     * 24:   OCTET STRING
     *	 :   }
     *
     * By replacing the 5th byte (0x04) with 0x16 we would have;
     *	      8:     IA5String 'TIGER192'
     */
static byte tiger192_asn_array[] = { /* this is wrong... but compatible with GnuPG */
    0x30,0x28,0x30,0x0c,0x04,0x08,0x54,0x49,0x47,
    0x45,0x52,0x31,0x39,0x32,0x05,0x00,0x04,0x18 };

static byte haval_5_160_asn_array[] = { /* this is wrong... */
    0x30,0x27,0x30,0x0f,0x04,0x0b,0x48,0x41,0x56,
    0x41,0x4C,0x2D,0x35,0x2D,0x31,0x36,0x30,0x05,0x00,0x04,0x14 };

static byte haval_5_256_asn_array[] = { /* this is wrong... */
    0x30,0x33,0x30,0x0f,0x04,0x0b,0x48,0x41,0x56,
    0x41,0x4C,0x2D,0x35,0x2D,0x32,0x35,0x36,0x05,0x00,0x04,0x20 };

/* This many bytes from the end, there's a zero byte */
#define ASN_ZERO_END 3
